### Empty test file for chkbuild ###

# If chkbuild fails with `test-all`, each child file/directory is
# executed individually, but test-unit fails if there are no test
# files in the specified directory.
