require_relative '../../../spec_helper'
require 'uri'

describe "URI::MailTo#to" do
  it "needs to be reviewed for spec completeness"
end

describe "URI::MailTo#to=" do
  it "needs to be reviewed for spec completeness"
end
